package future_good

import (
	"errors"
	"eta_gn/eta_index_lib/global"
	"eta_gn/eta_index_lib/utils"
	"time"
)

type FutureGoodEdbInfo struct {
	FutureGoodEdbInfoId int       `gorm:"primaryKey;autoIncrement;column:future_good_edb_info_id"`
	FutureGoodEdbCode   string    `gorm:"column:future_good_edb_code" description:"期货指标code"`
	FutureGoodEdbName   string    `gorm:"column:future_good_edb_name" description:"期货指标名称"`
	FutureGoodEdbNameEn string    `gorm:"column:future_good_edb_name_en" description:"期货指标英文名称"`
	ParentId            int       `gorm:"column:parent_id" description:"上级期货id"`
	RegionType          string    `gorm:"column:region_type" description:"交易所来源，海外还是国内"`
	Exchange            string    `gorm:"column:exchange" description:"所属交易所"`
	FutureGoodEdbType   int       `gorm:"column:future_good_edb_type" description:"指标类型,1:年月是固定的合约；2：只有M+N期的合约，未固定年月"`
	DateSourceId        int       `gorm:"column:date_source_id" description:"画图时，日期来源的指标id"`
	Year                int       `gorm:"column:year" description:"所属年份"`
	Month               int       `gorm:"column:month" description:"所属月份"`
	StartDate           string    `gorm:"column:start_date" description:"起始日期"`
	EndDate             string    `gorm:"column:end_date" description:"终止日期"`
	MinValue            float64   `gorm:"column:min_value" description:"最小值"`
	MaxValue            float64   `gorm:"column:max_value" description:"最大值"`
	LatestValue         float64   `gorm:"column:latest_value" description:"数据最新的值"`
	LatestDate          time.Time `gorm:"column:latest_date" description:"数据最新的日期"`
	ServerUrl           string    `gorm:"column:server_url" description:"服务器地址"`
	TerminalCode        string    `gorm:"column:terminal_code" description:"终端编码，用于配置在机器上"`
	CreateTime          time.Time `gorm:"column:create_time"`
	ModifyTime          time.Time `gorm:"column:modify_time"`
}

// GetFutureGoodEdbInfo 期货指标
func GetFutureGoodEdbInfo(edbInfoId int) (item *FutureGoodEdbInfo, err error) {

	sql := `SELECT * FROM future_good_edb_info WHERE future_good_edb_info_id = ? `
	sql += ` ORDER BY future_good_edb_info_id DESC `
	err = global.DEFAULT_DmSQL.Raw(sql, edbInfoId).First(&item).Error
	return
}

// GetFutureGoodEdbInfoByCode 根据指标code获取指标信息
func GetFutureGoodEdbInfoByCode(edbCode string) (item *FutureGoodEdbInfo, err error) {

	sql := `SELECT * FROM future_good_edb_info  WHERE future_good_edb_code = ? limit 1 `
	err = global.DEFAULT_DmSQL.Raw(sql, edbCode).First(&item).Error
	return
}

// GetFutureGoodEdbInfoList 获取指标数据列表
func GetFutureGoodEdbInfoList(condition string, pars []interface{}) (list []*FutureGoodEdbInfo, err error) {

	sql := `SELECT * FROM future_good_edb_info WHERE 1=1 `
	if condition != "" {
		sql += condition
	}
	sql += `ORDER BY future_good_edb_info_id DESC `
	err = global.DEFAULT_DmSQL.Raw(sql, pars...).Scan(&list).Error
	return
}

// GetAllFutureGoodEdbInfoList 获取指标数据列表
func GetAllFutureGoodEdbInfoList() (list []*FutureGoodEdbInfo, err error) {

	sql := `SELECT * FROM future_good_edb_info  ORDER BY future_good_edb_info_id DESC `
	err = global.DEFAULT_DmSQL.Raw(sql).Scan(&list).Error
	return
}

// AddFutureGoodEdbInfo 添加期货数据库指标
func AddFutureGoodEdbInfo(item *FutureGoodEdbInfo) (err error) {

	err = global.DEFAULT_DmSQL.Create(item).Error
	return
}

// GetChildFutureGoodEdbInfoListByParentId 根据父级ID获取下面所有的指标数据列表
func GetChildFutureGoodEdbInfoListByParentId(parentId int) (list []*FutureGoodEdbInfo, err error) {

	sql := `SELECT * FROM future_good_edb_info WHERE parent_id = ? ORDER BY future_good_edb_info_id ASC `
	err = global.DEFAULT_DmSQL.Raw(sql, parentId).Scan(&list).Error
	return
}

// Update 更新指标基础信息
func (FutureGoodEdbInfo *FutureGoodEdbInfo) Update(cols []string) (err error) {
	//err = o.Update(FutureGoodEdbInfo, cols...)
	err = global.DEFAULT_DmSQL.Model(FutureGoodEdbInfo).Select(cols).Updates(FutureGoodEdbInfo).Error
	return
}

// UnifiedModifyEdbInfoMaxAndMinInfo 统一修改指标的最大最小值
func UnifiedModifyEdbInfoMaxAndMinInfo(futureGoodEdbInfo *FutureGoodEdbInfo) (err error, errMsg string) {
	// 修改最大最小值
	maxAndMinItem, err := GetEdbInfoMaxAndMinInfo(futureGoodEdbInfo.FutureGoodEdbCode)
	if err != nil {
		if err.Error() == utils.ErrNoRow() {
			err = nil
			return
		}

		errMsg = "刷新期货指标失败！"
		err = errors.New("获取期货指标最大最小值失败，err:" + err.Error())
		return
	}

	if maxAndMinItem != nil {

		sql := ` UPDATE future_good_edb_info SET start_date=?,end_date=?,min_value=?,max_value=?,latest_date=?,latest_value=?,modify_time=NOW() WHERE future_good_edb_info_id=? `
		err = global.DEFAULT_DmSQL.Exec(sql, maxAndMinItem.MinDate, maxAndMinItem.MaxDate, maxAndMinItem.MinValue, maxAndMinItem.MaxValue, maxAndMinItem.MaxDate, maxAndMinItem.LatestValue, futureGoodEdbInfo.FutureGoodEdbInfoId).Error
		if err != nil {
			errMsg = "刷新指标失败！"
			err = errors.New("修改指标最大最小值失败，err:" + err.Error())
			return
		}
	}
	return
}

// FutureGoodEdbInfoMaxAndMinInfo 指标最新数据记录结构体
type FutureGoodEdbInfoMaxAndMinInfo struct {
	MinDate     string  `description:"最小日期"`
	MaxDate     string  `description:"最大日期"`
	MinValue    float64 `description:"最小值"`
	MaxValue    float64 `description:"最大值"`
	LatestValue float64 `description:"最新值"`
	LatestDate  string  `description:"实际数据最新日期"`
}

// GetEdbInfoMaxAndMinInfo 获取指标的最新数据记录信息
func GetEdbInfoMaxAndMinInfo(futureGoodEdbCode string) (item *FutureGoodEdbInfoMaxAndMinInfo, err error) {

	sql := ``
	sql = ` SELECT MIN(data_time) AS min_date,MAX(data_time) AS max_date,MIN(close) AS min_value,MAX(close) AS max_value FROM future_good_edb_data WHERE future_good_edb_code=? `
	err = global.DEFAULT_DmSQL.Raw(sql, futureGoodEdbCode).First(&item).Error

	var latest_value float64
	sql = ` SELECT close AS latest_value FROM future_good_edb_data WHERE future_good_edb_code=? ORDER BY data_time DESC LIMIT 1 `
	err = global.DEFAULT_DmSQL.Raw(sql, futureGoodEdbCode).Scan(&latest_value).Error
	item.LatestValue = latest_value
	return
}
